Imports System.Data.SqlClient
Imports System.Data

Partial Class _Default
    Inherits System.Web.UI.Page

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        ' We check to see that this page loads without any postback on the page because it is unnessary to
        ' to run this code if there only is a postback on the page like a click on a button.

        If Page.IsPostBack = False Then

            ' We select all products from the "Products" table and sort the list by "ProductName" in ascending order. 
            ' The data for the products are supplied to the "ProductListRepeater" repeater control and we split up the
            ' product list in pages with next and previous links for navigation.

            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT ProductID, ProductName, Description, PriceExSaleTax, CAST(PriceExSaleTax * SaleTaxPercent as decimal(18,2)) As SaleTaxMoney, SaleTaxPercent, CAST(PriceExSaleTax * (1 + SaleTaxPercent) as decimal(18,2)) As TotalPrice FROM Products ORDER BY ProductName ASC"

                ' The Using block is used to call dispose (close) automatically even if there are an exception.
                Using cn As New SqlConnection(ConnString), _
                      sad As New SqlDataAdapter(), _
                      cmd As New SqlCommand(sql, cn)

                    cn.Open()

                    ' Assign the select command to the SqlDataAdapter

                    sad.SelectCommand = cmd

                    ' Create the data table with the name of "dt"

                    Using dt As DataTable = New DataTable()

                        ' Fill the data table "dt" with data from the SqlDataAdapter

                        sad.Fill(dt)

                        ' Populate the repeater control with the datatable 
                        Dim objPds As PagedDataSource = New PagedDataSource()
                        objPds.DataSource = dt.DefaultView

                        ' Indicate that the data should be paged 
                        objPds.AllowPaging = True

                        ' Set the pagesize 
                        objPds.PageSize = 10

                        ' The current page (curpage) is declared as an integer
                        Dim curpage As Int32

                        ' We set the page number for the current page from the page parameter in the url
                        ' or to 1 if there are no page parameter in the url

                        If Not IsNothing(Request.QueryString("page")) Then
                            curpage = Convert.ToInt32(Request.QueryString("page"))
                        Else
                            curpage = 1
                        End If

                        ' The current page index is equal to the current page minus 1 and we need to know this
                        ' to show the right page to the user.

                        objPds.CurrentPageIndex = curpage - 1

                        ' We set the text in the middle to show the current pagenumber and the last pagenumber

                        lblCurrentPage.Text = "| Page: " + curpage.ToString() + " of " + objPds.PageCount.ToString() + " |"

                        ' This is code for the link to the previous page, we don't show this link on the first page

                        If Not objPds.IsFirstPage Then
                            lnkPrev.Visible = True
                            lnkPrev.NavigateUrl = "~/Default.aspx?" + "page=" + CStr(curpage - 1)
                        End If

                        ' This is code for the link to the next page, we don't show this link on the last page

                        If Not objPds.IsLastPage Then
                            lnkNext.Visible = True
                            lnkNext.NavigateUrl = "~/Default.aspx?" + "page=" + CStr(curpage + 1)
                        End If

                        ' This code is used to bind data to the repeater control

                        ProductListRepeater.DataSource = objPds
                        ProductListRepeater.DataBind()

                    End Using
                End Using
            Catch ex As Exception
                Response.Write(ex.Message)
            End Try
        End If
    End Sub

    Public Function GenerateURL(ByVal Product As Object) As String

        'Create URL for each product link
        Dim strProdUrl As String = "~/Product.aspx?Pid=" & Product

        Return strProdUrl
    End Function
End Class
