Imports System.Data
Imports System.Data.SqlClient

Partial Class Orders
    Inherits System.Web.UI.Page

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        ' We check to see that this page loads without any postback on the page because it is unnessary to
        ' to run this code if there only is a postback on the page.

        If Page.IsPostBack = False Then

            ' We check to see if there exists an FormAuthenticationTicket for the user.

            If Request.IsAuthenticated = True Then
                Dim id As FormsIdentity = CType(User.Identity, FormsIdentity)
                Dim ticket As FormsAuthenticationTicket = id.Ticket

                ' The UserData information from the FormsAuthenticationTicket contains the
                ' "CustomerID" of the signed in user and this value is stored in the hidden field "HiddenCustomerID".

                HiddenCustomerID.Value = ticket.UserData

                ' We call this subroutine to create the order list

                LoadOrderList()

            Else
                Response.Redirect("Not-signed-in.aspx")
            End If
        End If
    End Sub

    Protected Sub LoadOrderList()

        ' We select all orders from the "Orders" table where the CustomerID equals the CustomerID stored in the
        ' hidden field "HiddenCustomerID" and binds the data to the "OrderListRepeater" control.

        Try
            Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
            Dim sql As String = "SELECT * FROM Orders WHERE CustomerID = @CustomerID ORDER BY OrderID ASC"

            ' The Using block is used to call dispose (close) automatically even if there are an exception.
            Using cn As New SqlConnection(ConnString), _
                  cmd As New SqlCommand(sql, cn)
                cmd.Parameters.Add("@CustomerID", SqlDbType.BigInt).Value = HiddenCustomerID.Value

                cn.Open()
                ' The Using block is used to call dispose (close) automatically even if there are an exception.
                Using reader As SqlDataReader = cmd.ExecuteReader()
                    OrderListRepeater.DataSource = reader
                    OrderListRepeater.DataBind()
                End Using

            End Using
        Catch ex As Exception
            Response.Write(ex.Message)
        End Try
    End Sub

    Protected Sub OrderListRepeater_ItemCommand(ByVal source As Object, ByVal e As System.Web.UI.WebControls.RepeaterCommandEventArgs) Handles OrderListRepeater.ItemCommand

        ' We pick up the order id for the selected order according to the command argument and store
        ' this value in a variable called "OrderID"

        Dim OrderID As String = (e.CommandArgument).ToString()

        ' We set visiblity for OrderListPanel and OrderPanel

        OrderListPanel.Visible = False
        OrderPanel.Visible = True

        ' We select the order from the Orders table that has the same OrderID as the value in the
        ' OrderID variable. We use an SQL datareader to get data and passes the data from one single row
        ' (CommandBehavior.SingleRow) to labels on the Orders.aspx webpage.

        Try
            Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
            Dim sql As String = "SELECT * FROM Orders WHERE OrderID = @OrderID"

            ' The Using block is used to call dispose (close) automatically even if there are an exception.
            Using cn As New SqlConnection(ConnString), _
                  cmd As New SqlCommand(sql, cn)
                cmd.Parameters.Add("@OrderID", SqlDbType.BigInt).Value = OrderID

                cn.Open()
                ' The Using block is used to call dispose (close) automatically even if there are an exception.
                Using reader As SqlDataReader = cmd.ExecuteReader(CommandBehavior.SingleRow)
                    While reader.Read()

                        lbOrderID.Text = reader("OrderID").ToString()
                        lbOrderDate.Text = String.Format("{0:yyyy-MM-dd}", reader("OrderDate").ToString())
                        lbName.Text = reader("Company").ToString()
                        lbCustomerNumber.Text = reader("CustomerID").ToString()
                        lbAttention.Text = reader("Attention").ToString()
                        lbContact.Text = reader("Contact").ToString()
                        lbAdress.Text = reader("Adress").ToString()
                        lbPostalCode.Text = reader("PostalCode").ToString()
                        lbCity.Text = reader("City").ToString()
                        lbCountry.Text = reader("Country").ToString()

                    End While
                End Using

            End Using
        Catch ex As Exception
            Response.Write(ex.Message)
        End Try

        ' When we have selected a order we want to select the product rows for this order in the OrdersProducts
        ' table and fill the "ProductRowRepeater" with these rows. In our SELECT statement we have a INNER JOIN
        ' statement to get the product name from the "Products" table and a calculation for row sum.

        Try
            Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
            Dim sql As String = "SELECT O.ProductID, P.ProductName, O.SaleTaxPercent * 100 As VATP , O.Quantity, O.PriceExSaleTax, O.Quantity * O.PriceExSaleTax As RowSum FROM OrdersProducts As O INNER JOIN Products As P ON P.ProductID = O.ProductID WHERE OrderID = @OrderID GROUP BY O.ProductID, P.ProductName, O.SaleTaxPercent, O.Quantity, O.PriceExSaleTax ORDER BY ProductID ASC"

            ' The Using block is used to call dispose (close) automatically even if there are an exception.
            Using cn As New SqlConnection(ConnString), _
                  cmd As New SqlCommand(sql, cn)
                cmd.Parameters.Add("@OrderID", SqlDbType.BigInt).Value = OrderID

                cn.Open()
                Using reader As SqlDataReader = cmd.ExecuteReader()
                    ProductRowRepeater.DataSource = reader
                    ProductRowRepeater.DataBind()
                End Using

            End Using
        Catch ex As Exception
            Response.Write(ex.Message)
        End Try

        ' Call subroutine to calculate total sums

        CalculateOrderSums()

    End Sub

    Protected Sub linkOrderList_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles linkOrderList.Click

        ' Set visiblity for OrderListPanel and OrderPanel

        OrderListPanel.Visible = True
        OrderPanel.Visible = False

    End Sub

    Protected Sub CalculateOrderSums()

        ' We declare two variables that we will use for our calculations of total sums.

        Dim PriceExVat As Decimal = 0
        Dim VatMoney As Decimal = 0

        ' We iterate through each row in the "ProductRowRepeater and add values to our two variables

        For Each RepeaterRow As RepeaterItem In ProductRowRepeater.Items

            Dim VatObj As Literal = DirectCast(RepeaterRow.FindControl("ltVAT"), Literal)
            Dim RowSumObj As Literal = DirectCast(RepeaterRow.FindControl("ltRowSum"), Literal)

            PriceExVat += Convert.ToDecimal(RowSumObj.Text)
            VatMoney += Convert.ToDecimal(RowSumObj.Text) * (Convert.ToDecimal(VatObj.Text) / 100)
        Next

        ' We add the sums to labels and calculate the totalsum as price excluding VAT plus VAT in money

        lblPriceTotal.Text = Convert.ToString(PriceExVat)
        lblVatTotal.Text = Convert.ToString(VatMoney)
        lblTotalSum.Text = Convert.ToString(PriceExVat + VatMoney)

    End Sub

End Class
