Imports System.Data.SqlClient
Imports System.Data
Imports System.Web.Security

Partial Class Product
    Inherits System.Web.UI.Page

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        ' We check to see that this page loads without any postback on the page because it is unnessary to
        ' to run this code if there only is a postback on the page like a click on a button.

        If Page.IsPostBack = False Then

            ' When this webpage loads we request the product id from the parameter "Pid" in the url and
            ' places this value in the hidden field "HiddenProductID".

            HiddenProductID.Value = Convert.ToString(Request.QueryString("Pid"))

            ' We select the product that has the product id that is saved in the "HiddenProductID" field and
            ' binds the data from the select statement to the controls on "Product.aspx".

            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT ProductID, ProductName, Description, PriceExSaleTax, CAST(PriceExSaleTax * SaleTaxPercent as decimal(18,2)) As SaleTaxMoney, SaleTaxPercent, CAST(PriceExSaleTax * (1 + SaleTaxPercent) as decimal(18,2)) As TotalPrice FROM Products WHERE ProductID = @ProductID"

                ' The Using block is used to call dispose (close) automatically even if there are an exception.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@ProductID", SqlDbType.VarChar, 20).Value = HiddenProductID.Value

                    cn.Open()
                    Using reader As SqlDataReader = cmd.ExecuteReader(CommandBehavior.SingleRow)
                        While reader.Read()
                            HiddenProductID.Value = reader("ProductID").ToString()
                            ltProductName.Text = reader("ProductName").ToString()
                            ltDescription.Text = reader("Description").ToString()
                            ltPriceExSaleTax.Text = reader("PriceExSaleTax").ToString()
                            ltSaleTax.Text = reader("SaletaxMoney").ToString()
                            ltSaleTaxPercent.Text = String.Format("{0:P}", reader("SaletaxPercent"))
                            ltTotalPrice.Text = reader("TotalPrice").ToString()
                        End While
                    End Using

                End Using
            Catch ex As Exception
                Response.Write(ex.Message)
            End Try
        End If
    End Sub

    Protected Sub btnBuy_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnBuy.Click

        ' We first check to see if the user has signed in and if the user not is signed in we redirect him
        ' to the "Not-signed-in.aspx" webpage.

        If Request.IsAuthenticated = True Then

            ' We declare one variable as int32 to be able to use the value in the quantity textbox.

            Dim AddNumberOfUnits As Int32

            ' We try to convert the text in the textbox "txtQuantity" to a Int32 with the use of TRYPARSE and if this
            ' operation succeed this value will be added to the "AddNumberOfUnits" variable. If the conversion not is
            ' a success the "AddNumberOfUnits" variable gets 0 as the value.

            Int32.TryParse(txtQuantity.Text, AddNumberOfUnits)

            ' We declare ShoppingCookie as a HttpCookie, selects the cookie with the name "CartCookie" and
            ' store this cookie in "ShoppingCookie". If we dont find any cookie with the name of "CartCookie"
            ' "ShoppingCookie will get "Nothing" as the value.
            
            Dim ShoppingCookie As HttpCookie
            ShoppingCookie = Request.Cookies.Get("CartCookie")

            ' We check to see if the HttpCookie with the name of CartCookie exists so that we not
            ' will get any null point exeptions if the HttpCookie does not exist. If the cookie
            ' exists we use this cookie and if the cookie does not exist we create a new cookie.

            If Not ShoppingCookie Is Nothing Then

                ' We check to see if the HttpCookie has keys, if the HttpCookie has keys we
                ' find the value (quantity) from the key that corresponds to the ProductID
                ' and add the supplied quantity to the old quantity. If the HttpCookie does
                ' not have keys we just add the supplied quantity to a key with the ProductID and
                ' deletes on key with the value of "Nothing" that are created in this case.

                If ShoppingCookie.HasKeys Then
                    Dim OldQty As Int32 = Convert.ToInt32(Request.Cookies("CartCookie")(HiddenProductID.Value))

                    ShoppingCookie.Values(HiddenProductID.Value) = Convert.ToString(OldQty + AddNumberOfUnits)
                    ShoppingCookie.Expires = Now.AddHours(3)
                    Response.Cookies.Add(ShoppingCookie)
                Else
                    ShoppingCookie.Values(HiddenProductID.Value) = Convert.ToString(AddNumberOfUnits)
                    ShoppingCookie.Values.Remove(Nothing)
                    ShoppingCookie.Expires = Now.AddHours(3)
                    Response.Cookies.Add(ShoppingCookie)
                End If

            Else

                ' If a CartCookie does not exist we create a new HttpCookie and add the 
                ' quantity to a key with the name of the ProductID.

                ShoppingCookie = New HttpCookie("CartCookie")
                ShoppingCookie.Values(HiddenProductID.Value) = Convert.ToString(AddNumberOfUnits)
                ShoppingCookie.Expires = Now.AddHours(3)
                Response.Cookies.Add(ShoppingCookie)
            End If

            ' Update the small shopping cart by calling a public method in the Start class for the masterpage.
            DirectCast(Me.Master, Start).LoadSmallCart()

        Else
            Response.Redirect("Not-signed-in.aspx")
        End If
    End Sub
End Class
