Imports System.Data
Imports System.Data.SqlClient
Imports System.Web.UI.WebControls

Partial Class shopping_cart
    Inherits System.Web.UI.Page

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        ' We check to see that this page loads without any postback on the page because it is unnessary to
        ' to run this code if there only is a postback on the page like a click on a button.

        If Page.IsPostBack = False Then

            ' We check to see if there exists an FormAuthenticationTicket for the user.

            If Request.IsAuthenticated = True Then
                Dim id As FormsIdentity = CType(User.Identity, FormsIdentity)
                Dim ticket As FormsAuthenticationTicket = id.Ticket

                ' We get the CustomerID from UserData in the FormsAuthenticationTicket and store this value
                ' in the hidden field "HiddenCustomerID".

                HiddenCustomerID.Value = Convert.ToString(ticket.UserData)

                ' We call this subroutine to create our shopping cart

                LoadShoppingCart()

            Else
                Response.Redirect("Not-signed-in.aspx")
            End If
        End If
    End Sub

    Protected Sub LoadShoppingCart()

        ' We create a DataTable (in memory) to store dat in and a DataRow so that we can add rows to the
        ' DataTable as we iterate through each product in the HttpCookie.

        Dim tblShoppingCart As DataTable = New DataTable("CartTable")
        Dim rwShoppingCart As DataRow

        ' We declare "num" to iterate through the HttpCookie and we declare "CartCookie" to be able to check for
        ' its existens and to get data from this HttpCookie.

        Dim num As Int32
        Dim CartCookie As HttpCookie = Request.Cookies.Get("CartCookie")

        ' We check to see if the HttpCookie with the name of CartCookie exists so that we not
        ' will get any null point exeptions if the HttpCookie does not exist. We also check if
        ' the "CartCookie" has any keys, if there are no keys there are no products in the
        ' shopping cart and then we will get an exeption if we try to selects products
        ' from the "Products" table.

        If Not CartCookie Is Nothing Then

            If CartCookie.HasKeys Then

                ' We add 5 columns to our DataTable, we need to add the columns before we iterate through each
                ' product so that we get only one set of columns :-).

                tblShoppingCart.Columns.Add("ProductIDC")
                tblShoppingCart.Columns.Add("ProductName")
                tblShoppingCart.Columns.Add("SaletaxPercent")
                tblShoppingCart.Columns.Add("QuantityC")
                tblShoppingCart.Columns.Add("PriceExSaleTax")

                ' We iterate through each post in the HttpCookie and get the ProductID and Quantity for
                ' each post. We use the ProductID to select data from the Products table.

                For num = 0 To CartCookie.Values.Count - 1
                    Dim ProductID As String = CartCookie.Values.AllKeys(num)
                    Dim Quantity As String = CartCookie.Values(num)

                    Try
                        Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                        Dim sql As String = "SELECT * FROM Products WHERE ProductID = @ProductID"

                        ' The Using block is used to call dispose (close) automatically even if there are an exception.
                        Using cn As New SqlConnection(ConnString), _
                              cmd As New SqlCommand(sql, cn)
                            cmd.Parameters.Add("@ProductID", SqlDbType.VarChar, 20).Value = ProductID

                            cn.Open()
                            ' The Using block is used to call dispose (close) automatically even if there are an exception.
                            Using reader As SqlDataReader = cmd.ExecuteReader(CommandBehavior.SingleRow)

                                While reader.Read()

                                    ' We add one row to the DataTable and fill each cell with data.

                                    rwShoppingCart = tblShoppingCart.NewRow
                                    rwShoppingCart(0) = ProductID
                                    rwShoppingCart(1) = reader("ProductName").ToString()
                                    rwShoppingCart(2) = Convert.ToDecimal(reader("SaletaxPercent")) * 100
                                    rwShoppingCart(3) = Quantity
                                    rwShoppingCart(4) = reader("PriceExSaleTax").ToString()
                                    tblShoppingCart.Rows.Add(rwShoppingCart)

                                End While
                            End Using

                        End Using
                    Catch ex As Exception
                        Response.Write(ex.Message)
                    End Try
                Next

                ' We fill our Repeater control with the data from the "tblShoppingCart" DataTable.

                CartRepeater.DataSource = tblShoppingCart
                CartRepeater.DataBind()

                ' Call a subroutine to calculate totals in the shopping cart

                CalculateCartSums()

            Else
                ' If the HttpCookie does not have any keys we update the shopping cart so that it does not show
                ' any rows.

                CartRepeater.DataSource = ""
                CartRepeater.DataBind()

                ' Call a subroutine to calculate totals in the shopping cart

                CalculateCartSums()

            End If
        End If
    End Sub

    Protected Sub CartRepeater_ItemCommand(ByVal source As Object, ByVal e As System.Web.UI.WebControls.RepeaterCommandEventArgs) Handles CartRepeater.ItemCommand

        ' This code runs when a user has clicked a remove button (X) on a row.
        ' We get the ProductID from the CommandArgument when the remove button is clicked.

        Dim ProductID As String = (e.CommandArgument).ToString()

        ' We declare the variable CartCookie to check if the HttpCookie "CartCookie" exits and to 
        ' be able to remove a key from it.

        Dim CartCookie As HttpCookie = Request.Cookies.Get("CartCookie")

        ' We check if the CartCookie exists and then removes the key that has the "ProductID" as its name
        ' and then we then reload the shopping cart so that our repeater gets updated.

        If Not CartCookie Is Nothing Then
            CartCookie.Values.Remove(ProductID)
            CartCookie.Expires = Now.AddHours(3)
            Response.Cookies.Add(CartCookie)

            ' Reload the shopping cart
            LoadShoppingCart()

            ' Update the small shopping cart by calling a public method in the Start class for the masterpage.
            DirectCast(Me.Master, Start).LoadSmallCart()

        End If
    End Sub

    Protected Sub btnUpdateCart_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnUpdateCart.Click

        ' We declare the variable CartCookie to check if the HttpCookie "CartCookie" exits and to 
        ' be able to remove a key from it.

        Dim CartCookie As HttpCookie = Request.Cookies.Get("CartCookie")

        ' We check to see if the HttpCookie with the name of CartCookie exists so that we not
        ' will get any null point exeptions if the HttpCookie does not exist.

        If Not CartCookie Is Nothing Then

            If CartCookie.HasKeys Then
                For Each RepeaterRow As RepeaterItem In CartRepeater.Items
                    Dim ProductObj As Literal = DirectCast(RepeaterRow.FindControl("ltProductID"), Literal)
                    Dim QuantityObj As TextBox = DirectCast(RepeaterRow.FindControl("txtQuantity"), TextBox)

                    CartCookie.Values(ProductObj.Text) = QuantityObj.Text
                    CartCookie.Expires = Now.AddHours(3)
                    Response.Cookies.Add(CartCookie)
                Next

            End If

            ' Recalculate the shopping cart

            CalculateCartSums()

            ' Update the small shopping cart by calling a public method in the Start class for the masterpage.
            DirectCast(Me.Master, Start).LoadSmallCart()

        End If

    End Sub

    Protected Sub btnCheckOut_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnCheckOut.Click

        ' When the user wants to check out and has clicked the "Check out" button we make the
        ' "CheckOutPanel" (includes textboxes for customer information) visible.

        CheckOutPanel.Visible = True

        ' We use the value for CustomerID that is stored in the hidden field "HiddenCustomerID" to get
        ' information on the customer in the "Customers" table and inserts this information in textboxes.
        ' We use an SqlDataReader to get the information and just want to select one single row (CommandBehavior.SingleRow).

        Try
            Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
            Dim sql As String = "SELECT * FROM Customers WHERE CustomerID = @CustomerID"

            ' The Using block is used to call dispose (close) automatically even if there are an exception.
            Using cn As New SqlConnection(ConnString), _
                  cmd As New SqlCommand(sql, cn)
                cmd.Parameters.Add("@CustomerID", SqlDbType.BigInt).Value = HiddenCustomerID.Value

                cn.Open()
                ' The Using block is used to call dispose (close) automatically even if there are an exception.
                Using reader As SqlDataReader = cmd.ExecuteReader(CommandBehavior.SingleRow)
                    While reader.Read()

                        txtCompany.Text = reader("Company").ToString()
                        txtOrganisationNumber.Text = reader("OrgNumber").ToString()
                        txtContact.Text = reader("Contact").ToString()
                        txtAttention.Text = reader("Attention").ToString()
                        txtAdress.Text = reader("Adress").ToString()
                        txtPostalCode.Text = reader("PostalCode").ToString()
                        txtCity.Text = reader("City").ToString()
                        txtCountry.Text = reader("Country").ToString()

                    End While
                End Using

            End Using
        Catch ex As Exception
            Response.Write(ex.Message)
        End Try
    End Sub

    Protected Sub btnOrder_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnOrder.Click

        ' We declare the variable "Identity" that will store the OrderID of the order that is added so that we can use
        ' this OrdeID number when we are to insert data in the table "OrderProduct" that has a many to many relationsship
        ' to the Order table. Check database diagrams to se the relationships between tables.

        Dim Identity As Int64 = 0

        ' We insert an order in the "Orders" table that has OrderID as a identity field that increments with 1.
        ' We have added ; SELECT SCOPE_IDENTITY() to the SQL statement to get the OrderID of the inserted order
        ' and use "ExecuteScalar()" instead of "ExecuteNonQuery()" in the SQL command.

        Try
            Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
            Dim sql As String = "INSERT INTO Orders (OrderDate, CustomerID, Company, OrgNumber, Contact, Attention, Adress, PostalCode, City, Country) VALUES (@OrderDate, @CustomerID, @Company, @OrgNumber, @Contact, @Attention, @Adress, @PostalCode, @City, @Country); SELECT SCOPE_IDENTITY()"

            ' The Using block is used to call dispose (close) automatically even if there are an exception.
            Using cn As New SqlConnection(ConnString), _
                  cmd As New SqlCommand(sql, cn)
                cmd.Parameters.Add("@OrderDate", SqlDbType.DateTime).Value = DateTime.Now.ToString()
                cmd.Parameters.Add("@CustomerID", SqlDbType.BigInt).Value = HiddenCustomerID.Value
                cmd.Parameters.Add("@Company", Data.SqlDbType.VarChar, 50).Value = txtCompany.Text
                cmd.Parameters.Add("@OrgNumber", Data.SqlDbType.VarChar, 15).Value = txtOrganisationNumber.Text
                cmd.Parameters.Add("@Contact", Data.SqlDbType.VarChar, 50).Value = txtContact.Text
                cmd.Parameters.Add("@Attention", Data.SqlDbType.VarChar, 50).Value = txtAttention.Text
                cmd.Parameters.Add("@Adress", Data.SqlDbType.VarChar, 50).Value = txtAdress.Text
                cmd.Parameters.Add("@PostalCode", Data.SqlDbType.VarChar, 10).Value = txtPostalCode.Text
                cmd.Parameters.Add("@City", Data.SqlDbType.VarChar, 50).Value = txtCity.Text
                cmd.Parameters.Add("@Country", Data.SqlDbType.VarChar, 50).Value = txtCountry.Text

                cn.Open()
                Identity = Convert.ToInt64(cmd.ExecuteScalar())

            End Using
        Catch Sqlex As SqlException
            Response.Write(Sqlex.Message)
        Catch ex As Exception
            Response.Write(ex.Message)
        End Try

        ' When we have inserted an order we want to insert the product rows for the order in the "OrdersProducts" table.
        ' We first check to see that the identity variable not has a blank value before we insert data to the
        ' "OrdersProducts" table.

        If Identity > 0 Then

            ' We iterate through each repeater row in the "CartRepeater" and insert every row in the
            ' "OrdersProduct" table.

            For Each RepeaterRow As RepeaterItem In CartRepeater.Items
                Dim ProductIDObj As Literal = DirectCast(RepeaterRow.FindControl("ltProductID"), Literal)
                Dim VATObj As Literal = DirectCast(RepeaterRow.FindControl("ltVAT"), Literal)
                Dim QuantityObj As TextBox = DirectCast(RepeaterRow.FindControl("txtQuantity"), TextBox)
                Dim PriceObj As Literal = DirectCast(RepeaterRow.FindControl("ltPrice"), Literal)

                Try
                    Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                    Dim sql As String = "INSERT INTO OrdersProducts (OrderID, ProductID, SaleTaxPercent, Quantity, PriceExSaleTax) VALUES (@OrderID, @ProductID, @SaleTaxPercent, @Quantity, @PriceExSaleTax)"

                    ' The Using block is used to call dispose (close) automatically even if there are an exception.
                    Using cn As New SqlConnection(ConnString), _
                          cmd As New SqlCommand(sql, cn)
                        cmd.Parameters.Add("@OrderID", SqlDbType.BigInt).Value = Identity
                        cmd.Parameters.Add("@ProductID", SqlDbType.VarChar, 20).Value = ProductIDObj.Text
                        cmd.Parameters.Add("@SaleTaxPercent", Data.SqlDbType.Decimal).Value = Convert.ToDecimal(VATObj.Text / 100)
                        cmd.Parameters.Add("@Quantity", Data.SqlDbType.Decimal).Value = Convert.ToDecimal(QuantityObj.Text)
                        cmd.Parameters.Add("@PriceExSaleTax", Data.SqlDbType.Decimal).Value = Convert.ToDecimal(PriceObj.Text)

                        cn.Open()
                        cmd.ExecuteNonQuery()

                    End Using
                Catch Sqlex As SqlException
                    Response.Write(Sqlex.Message)
                Catch ex As Exception
                    Response.Write(ex.Message)
                End Try
            Next

            ' Delete the HttpCookie, we declare the variable CartCookie to check if the HttpCookie "CartCookie" 
            ' exits and to be able to remove it.

            Dim CartCookie As HttpCookie = Request.Cookies.Get("CartCookie")

            ' We check if the CartCookie exists and then sets the expiration date to current DateTime minus one hour

            If Not CartCookie Is Nothing Then
                CartCookie.Expires = Now.AddHours(-1)
                Response.Cookies.Add(CartCookie)
            End If

            ' Redirect the user to the order list webpage

            Response.Redirect("Orders.aspx")

        End If

    End Sub

    Protected Sub CalculateCartSums()

        ' We declare variables to calculate totals for the shopping cart.

        Dim PriceTotal As Decimal = 0
        Dim VatTotal As Decimal = 0

        ' We iterate through each row in the "CartRepeater" and add values to our three variables

        For Each RepeaterRow As RepeaterItem In CartRepeater.Items
            Dim VatObj As Literal = DirectCast(RepeaterRow.FindControl("ltVAT"), Literal)
            Dim PriceExVat As Literal = DirectCast(RepeaterRow.FindControl("ltPrice"), Literal)
            Dim QuantityObj As TextBox = DirectCast(RepeaterRow.FindControl("txtQuantity"), TextBox)

            ' Additions for totals
            PriceTotal += Convert.ToDecimal(PriceExVat.Text) * Convert.ToDecimal(QuantityObj.Text)
            VatTotal += Convert.ToDecimal(PriceExVat.Text) * Convert.ToDecimal(QuantityObj.Text) * (Convert.ToDecimal(VatObj.Text) / 100)

        Next

        ' We set the totals to labels under our repeater control and calculate the totalsum

        lblPriceTotal.Text = Convert.ToString(PriceTotal)
        lblVatTotal.Text = Convert.ToString(VatTotal)
        lblTotalSum.Text = Convert.ToString(PriceTotal + VatTotal)

    End Sub
End Class
