Imports System.Data.SqlClient
Imports System.Data
Imports System.Web.Security

Partial Class Start
    Inherits System.Web.UI.MasterPage
    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        ' We check to see that this page loads by a new request for the page so that this code does not
        ' run when there is a postback like when a button is clicked.

        If Page.IsPostBack = False Then

            ' We check to see if there exists an FormAuthenticationTicket for the user.
            ' Hide the "SigninPanel" and show the "CustomerPanel" if the user is signed in
            ' on this website.

            If Request.IsAuthenticated = True Then
                Dim id As FormsIdentity = CType(Page.User.Identity, FormsIdentity)
                Dim ticket As FormsAuthenticationTicket = id.Ticket

                ' This is information from the FormsAuthenticationTicket

                lblUserName.Text = ticket.Name

                ' Set visibilities for panels

                SigninPanel.Visible = False
                CustomerPanel.Visible = True
                SmallCartPanel.Visible = True

                ' Call subroutine to load the small shopping cart

                LoadSmallCart()

            End If
        End If
    End Sub

    Protected Sub cmdLogin_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdLogin.Click

        ' This code will get executed when the "Sign in" button is clicked and the first thing that is
        ' done is to check if the username and password corresponds to a user in the "Customers" table.
        ' We are sending the username and password to the "ValidateUser" function in the If-statement
        ' and will then return an answer from this function, se this function after this subroutine.

        If ValidateUser(LoginUserName.Text, LoginUserPass.Text) Then

            ' We declare three variables that will be used to create a cookie for the signed in user.

            Dim ticket As FormsAuthenticationTicket
            Dim cookie As String
            Dim httpCookie As HttpCookie

            ' The FormsAuthenticationTicket is filled with data for version, name, issue date,
            ' expiration date, if the cookie are persistent or not, user data and cookie path.

            ticket = New FormsAuthenticationTicket(1, LoginUserName.Text, DateTime.Now(), _
            DateTime.Now.AddMinutes(100), chkPersistCookie.Checked, HiddenCustomerID.Value, "MyPage")

            ' The cookie is set to the encrypted ticket

            cookie = FormsAuthentication.Encrypt(ticket)

            ' The httpCookie gets a name and the value from the cookie

            httpCookie = New HttpCookie(FormsAuthentication.FormsCookieName(), cookie)

            ' We set the expiration date for the httpCookie if the ticket is persistent. A HttpCookie
            ' without an expire date will expire when the browser is closed.
            If (chkPersistCookie.Checked = True) Then
                httpCookie.Expires = ticket.Expiration
            End If

            ' Set the path for the HttpCookie.
            httpCookie.Path = FormsAuthentication.FormsCookiePath()

            ' Add the HttpCookie.
            Response.Cookies.Add(httpCookie)

            ' Redirect the user to the Default page so that the right panels are visible
            Response.Redirect("Default.aspx")

        Else
            lblMessage.Text = "* Incorrect password or e-mail"
        End If

    End Sub

    Private Function ValidateUser(ByVal userName As String, ByVal passWord As String) As Boolean

        ' This is a check that is done for the entered user name, i has to be between 1 and 80
        ' characters. If this check is false this function will return a "False" statement and the
        ' code below will not be executed.

        If ((userName.Length = 0) Or (userName.Length > 80)) Then
            System.Diagnostics.Trace.WriteLine("[ValidateUser] Input validation of userName failed.")
            Return False
        End If

        ' This is a check that is done for the entered password, i has to be between 1 and 25
        ' characters. If this check is false this function will return a "False" statement and the
        ' code below will not be executed.

        If ((passWord.Length = 0) Or (passWord.Length > 25)) Then
            System.Diagnostics.Trace.WriteLine("[ValidateUser] Input validation of passWord failed.")
            Return False
        End If

        ' We receive the userName and passWord as variables when this function is called and we 
        ' create the variable "lookupPassword" as a string to store the password that is selected from
        ' the database. The hidden field "HiddenCustomer" that should store the customer id of the customer
        ' are set to Blank at the start. The passWord that is passed to this function is encrypted with SHA1 
        ' because the password that is stored in the table "Customers" are encrypted with SHA1 and therefore 
        ' have to do this encryption to compare the entered password with the stored password in the database.

        Dim lookupPassword As String = Nothing
        HiddenCustomerID.Value = ""
        Dim passwordHash As String = FormsAuthentication.HashPasswordForStoringInConfigFile(passWord, "SHA1")

        ' This code is used to select the password and customer id from the "Customers" table according to
        ' the supplied username in the "sign in form".

        Try
            Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
            Dim SelectUser As String = "SELECT CustomerID, Password FROM Customers WHERE Username = @Username"

            ' The Using block is used to call dispose (close) automatically even if there are an exception.
            Using cn As New SqlConnection(ConnString), _
                  cmd As New SqlCommand(SelectUser, cn)
                cmd.Parameters.Add("@Username", SqlDbType.VarChar, 80).Value = LoginUserName.Text

                cn.Open()

                ' We use SqlDataReader and just want to select one single row. The CustomerID are
                ' supplied to the hidden field "HiddenCustomerID" and the Password are supplied to the
                ' "lookupPassword" string.

                ' The Using block is used to call dispose (close) automatically even if there are an exception.
                Using reader As SqlDataReader = cmd.ExecuteReader(System.Data.CommandBehavior.SingleRow)
                    While reader.Read()
                        HiddenCustomerID.Value = reader("CustomerID").ToString()
                        lookupPassword = reader("Password").ToString()
                    End While
                End Using

            End Using
        Catch ex As Exception
            System.Diagnostics.Trace.WriteLine("[ValidateUser] Exception " & ex.Message)
        End Try

        ' If no password is found this function will return false.

        If (lookupPassword Is Nothing) Then

            ' You could write failed login attempts here to the event log for additional security.

            Return False
        End If

        ' Compare lookupPassword and passwordHash by using a case-sensitive comparison.

        Return (String.Compare(lookupPassword, passwordHash, False) = 0)

    End Function
    Protected Sub btnSignOut_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnSignOut.Click

        ' This code is executed when the user clicks the "Sign out" button and deletes the cookie and the ticket
        ' for the customer.

        FormsAuthentication.SignOut()
        FormsAuthentication.RedirectToLoginPage()

    End Sub

    Public Sub LoadSmallCart()

        ' We declare "num" to iterate through the HttpCookie and we declare "CartCookie" to be able to check for
        ' its existens and to get data from this HttpCookie.

        Dim num As Int32
        Dim CartCookie As HttpCookie = Request.Cookies.Get("CartCookie")
        ltCartItem.Text = ""

        ' We check to see if the HttpCookie with the name of CartCookie exists so that we not
        ' will get any null point exeptions if the HttpCookie does not exist. We also check if
        ' the "CartCookie" has any keys, if there are no keys there are no products in the
        ' shopping cart and then we create a message for this.

        If Not CartCookie Is Nothing Then

            If CartCookie.HasKeys Then

                ' We iterate through each post in the HttpCookie and get the ProductID and Quantity for
                ' each post. We update the literal control "ltCartItem" with data from each row in the HttpCookie.

                For num = 0 To CartCookie.Values.Count - 1
                    ltCartItem.Text += CartCookie.Values.AllKeys(num) & " (" & CartCookie.Values(num) & ")" & "<br />"
                Next
            Else
                ltCartItem.Text = "There are no items in your shopping cart."
            End If
        Else
            ltCartItem.Text = "There are no items in your shopping cart."
        End If
    End Sub
End Class

